package ru.lifepay.checkout.demo.paymentterminal

import android.app.Activity
import ru.lifepay.checkout.demo.DefaultResponseHandler
import ru.lifepay.checkout.demo.Actions
import ru.lifepay.checkout.demo.getErrorMessageWithCode
import ru.lifepay.common.extensions.GSONConverter
import ru.lifepay.periphery.models.paymentterminals.PTEOperations
import ru.lifepay.periphery.models.paymentterminals.PTSummaryShiftReport
import ru.lifepay.periphery.models.paymentterminals.PTTransaction
import java.io.Serializable

class PTResponseHandler {

    fun handleSuccessResponse(response: DefaultResponseHandler.Response.Success) {
        val requestId = requireNotNull(response.extras.getString("request_id"))
        when(response.action) {
            Actions.PT_PAY -> {
                val data = requireNotNull(response.extras.getString("data"))
                val transaction = GSONConverter.fromJSON(data, PTTransaction::class.java)

                /**
                 * Сохраняем транзакцию оплаты,
                 * чтобы иметь возможность сделать по ней возврат в будущем
                 */
                requireNotNull(PTOrderHolder.currentOrder)
                    .setPaymentTransaction(transaction)
            }
            Actions.PT_REFUND -> {
                val data = requireNotNull(response.extras.getString("data"))
                val transaction = GSONConverter.fromJSON(data, PTTransaction::class.java)

                /**
                 * Сохраняем транзакцию возврата.
                 *
                 * Некоторые терминалы могут производить операцию отмены при вызове возврата
                 * При соблюдении определенных условий
                 * Одним из которых является выполнение операции возврата до проведения сверки итогов
                 */
                requireNotNull(PTOrderHolder.currentOrder)
                    .setRefundOrCancellationTransaction(transaction)
            }
            Actions.PT_CANCEL -> {
                val data = requireNotNull(response.extras.getString("data"))
                val transaction = GSONConverter.fromJSON(data, PTTransaction::class.java)

                /**
                 * Сохраняем транзакцию отмены оплаты.
                 */
                requireNotNull(PTOrderHolder.currentOrder)
                    .setRefundOrCancellationTransaction(transaction)
            }
            Actions.PT_GET_RESULT -> {
                val data = requireNotNull(response.extras.getString("data"))
                val transaction = GSONConverter.fromJSON(data, PTTransaction::class.java)

                /**
                 * Сохраняем полученную транзакцию.
                 */
                requireNotNull(PTOrderHolder.currentOrder).run {
                    when(transaction.operation) {
                        PTEOperations.PAYMENT -> {
                            setPaymentTransaction(transaction)
                        }
                        PTEOperations.REFUND,
                        PTEOperations.CANCELLATION -> {
                            setRefundOrCancellationTransaction(transaction)
                        }
                        else -> {}
                    }
                }
            }
            Actions.PT_CLOSE_SHIFT -> {
                val data = requireNotNull(response.extras.getString("data"))
                val report = GSONConverter.fromJSON(data, PTSummaryShiftReport::class.java)

                PTReportHolder.lastSummaryShiftReport = report
            }
            else -> {
                //Неизвестный action
            }
        }
    }

    interface GetResultCallback {
        fun getOperationResult()
        fun createNewOperation()
    }

    fun handleErrorResponse(
        response: DefaultResponseHandler.Response.Error,
        activity: Activity,
        showErrorCallback: (message: String?) -> Unit,
        showGetResultDialogCallback: ((errorMessage: String, callback: GetResultCallback) -> Unit)?
    ) {
        val requestId = requireNotNull(response.extras.getString("request_id"))
        val errorType = response.extras.getString("error_type")

        when(response.action) {
            Actions.PT_PAY,
            Actions.PT_REFUND,
            Actions.PT_CANCEL,
            Actions.PT_GET_RESULT  -> {
                if (errorType == "unknown_result_error" || errorType == "duplicate_request_error") {
                    /**
                     * Результат транзакции неизвестен. (unknown_result_error)
                     * Либо запрос был сделан запрос с тем же request_id (duplicate_request_error)
                     * Можно попытаться получить результат операции при помощи запроса результата операции.
                     */
                    val metadata = response.extras.getSerializable("metadata")
                    showGetResultDialog(
                        requestId = requestId,
                        metadata = metadata,

                        errorMessage = response.getErrorMessageWithCode(),
                        activity = activity,
                        showErrorCallback = showErrorCallback,
                        showGetResultDialogCallback = showGetResultDialogCallback
                    )
                    return
                } else {
                    //т.к. результат транзакции известен (ошибка), то обновляем request_id для следующего запроса
                    updateRequestIdForPaymentTerminalOperation(requestId)
                }
            }
            Actions.PT_CLOSE_SHIFT -> {}

            //Другие действия
            else -> {}
        }

        showErrorCallback.invoke(response.getErrorMessageWithCode())
    }

    private fun showGetResultDialog(
        requestId: String,
        metadata: Serializable?,

        errorMessage: String,
        activity: Activity,
        showErrorCallback: (message: String?) -> Unit,
        showGetResultDialogCallback: ((errorMessage: String, callback: GetResultCallback) -> Unit)?
    ) {
        showGetResultDialogCallback?.invoke(errorMessage, object : GetResultCallback {
            override fun getOperationResult() {
                PTRequest(
                    activity = activity,
                    errorCallback = showErrorCallback
                ).getResult(requestId, metadata)
            }

            override fun createNewOperation() {
                //Обновляем request_id, чтобы не получать duplicate_request_error
                updateRequestIdForPaymentTerminalOperation(requestId)
            }
        })
    }

    private fun updateRequestIdForPaymentTerminalOperation(requestId: String) {
        val order = requireNotNull(PTOrderHolder.currentOrder)
        when (requestId) {
            order.paymentTransactionRequestId -> order.updatePaymentTransactionRequestId()
            order.refundTransactionRequestId -> order.updateRefundTransactionRequestId()
            order.cancellationTransactionRequestId -> order.updateCancellationTransactionRequestId()
        }
    }
}