package ru.lifepay.checkout.demo.ui

import android.annotation.SuppressLint
import android.content.Intent
import android.content.pm.ActivityInfo
import android.graphics.Typeface
import android.os.Bundle
import android.view.MenuItem
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import ru.lifepay.checkout.demo.DefaultResponseHandler
import ru.lifepay.checkout.demo.R
import ru.lifepay.checkout.demo.getNoResponseErrorMessage
import ru.lifepay.checkout.demo.paymentterminal.PTOrder
import ru.lifepay.checkout.demo.paymentterminal.PTRequest
import ru.lifepay.checkout.demo.paymentterminal.PTOrderHolder
import ru.lifepay.checkout.demo.paymentterminal.PTResponseHandler
import ru.lifepay.checkout.demo.util.Util
import ru.lifepay.checkout.demo.util.Logger
import ru.lifepay.checkout.demo.util.getCompactRepresentation
import ru.lifepay.checkout.demo.util.intentToString

class PaymentTerminalActivity : AppCompatActivity() {

    lateinit var paymentPriceEditText: EditText
    lateinit var clearDataButton: Button

    lateinit var payButton: Button
    lateinit var refundButton: Button
    lateinit var cancelButton: Button

    lateinit var paymentContentTextView: TextView
    lateinit var refundOrCancellationContentTextView: TextView

    lateinit var lastIntentResultTextView: TextView


    @SuppressLint("SourceLockedOrientationActivity")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT
        setContentView(R.layout.activity_payment_terminal)

        paymentPriceEditText = findViewById(R.id.paymentPriceEditText)
        clearDataButton = findViewById(R.id.clearDataButton)
        payButton = findViewById(R.id.payButton)
        refundButton = findViewById(R.id.refundButton)
        cancelButton = findViewById(R.id.cancelButton)
        paymentContentTextView = findViewById(R.id.paymentContentTextView)
        refundOrCancellationContentTextView = findViewById(R.id.refundOrCancellationContentTextView)
        lastIntentResultTextView = findViewById(R.id.lastIntentResultTextView)

        this.supportActionBar?.apply {
            title = "Платежный терминал"
            setDisplayHomeAsUpEnabled(true)
            setDisplayShowHomeEnabled(true)
        }

        paymentContentTextView.typeface = Typeface.MONOSPACE
        refundOrCancellationContentTextView.typeface = Typeface.MONOSPACE

        payButton.setOnClickListener {
            val order = PTOrderHolder.currentOrder ?: run {
                val amount = getEnteredAmount()
                if (amount == null) {
                    Util.showError(this, "Введите сумму в копейках")
                    return@setOnClickListener
                }
                PTOrder(amount).also {
                    PTOrderHolder.currentOrder = it
                    updateUI()
                }
            }

            Util.hideKeyboard(this, paymentPriceEditText)

            createPaymentTerminal().pay(
                requestId = order.paymentTransactionRequestId,
                metadata = hashMapOf("key_1" to "value_1", "key_2" to "value_2"),
                amount = order.amount
            )
        }

        refundButton.setOnClickListener {
            val order = requireNotNull(PTOrderHolder.currentOrder)
            createPaymentTerminal().refund(
                requestId = order.refundTransactionRequestId,
                primaryTransaction = requireNotNull(order.paymentTransaction),
                amount = order.amount
            )
        }

        cancelButton.setOnClickListener {
            val order = requireNotNull(PTOrderHolder.currentOrder)
            createPaymentTerminal().cancel(
                requestId = order.cancellationTransactionRequestId,
                primaryTransaction = requireNotNull(order.paymentTransaction),
                amount = order.amount
            )
        }

        clearDataButton.setOnClickListener {
            paymentPriceEditText.text = null
            PTOrderHolder.currentOrder = null
            updateUI()
        }

        updateUI()
    }

    private fun getEnteredAmount(): Long? {
        return paymentPriceEditText.text.toString().toLongOrNull()
    }

    private fun createPaymentTerminal() = PTRequest(
        activity = this,
        errorCallback = Util.showErrorCallback(this))

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        Logger.logResponse(requestCode, resultCode, data)

        val response = DefaultResponseHandler.handleActivityResult(
            requestCode = requestCode,
            resultCode = resultCode,
            data = data
        )

        val responseHandler = PTResponseHandler()
        when (response) {
            is DefaultResponseHandler.Response.Success -> {
                responseHandler.handleSuccessResponse(response)
            }
            is DefaultResponseHandler.Response.Error -> {
                responseHandler.handleErrorResponse(
                    response = response,

                    activity = this,
                    showErrorCallback = Util.showErrorCallback(this),
                    showGetResultDialogCallback = this::showErrorWithGetResult
                )
            }
            is DefaultResponseHandler.Response.NoResponse -> {
                Util.showError(this, response.getNoResponseErrorMessage())
            }
        }
        updateUI()
        lastIntentResultTextView.text = "Результат последнего запроса:\n${data.intentToString()}"
    }

    private fun showErrorWithGetResult(
        errorMessage: String,
        getResultCallback: PTResponseHandler.GetResultCallback
    ) {
        val builder = AlertDialog.Builder(this)
        builder.setTitle("Ошибка")
            .setMessage("${errorMessage}\nПолучить результат операции?")
            .setCancelable(true)
            .setPositiveButton("Получить результат операции") { _, _ ->
                getResultCallback.getOperationResult()
            }
            .setNegativeButton("Создать новую операцию") { _, _ ->
                getResultCallback.createNewOperation()
            }
        val alert = builder.create()
        alert.show()
    }

    private fun updateUI() {
        val order = PTOrderHolder.currentOrder

        if (order == null) {
            paymentPriceEditText.text = null
            paymentPriceEditText.isEnabled = true
            clearDataButton.visibility = View.GONE
            payButton.visibility = View.VISIBLE
            refundButton.visibility = View.GONE
            cancelButton.visibility = View.GONE
            paymentContentTextView.text = ""
            refundOrCancellationContentTextView.text = ""
            lastIntentResultTextView.text = ""
            Util.hideKeyboard(this, paymentPriceEditText)
        } else {
            paymentPriceEditText.setText(order.amount.toString())
            paymentPriceEditText.isEnabled = false
            clearDataButton.visibility = View.VISIBLE

            order.paymentTransaction.also { paymentTransaction ->
                payButton.visibility = if (paymentTransaction == null) View.VISIBLE else View.GONE
                paymentContentTextView.text = paymentTransaction?.getCompactRepresentation()
                val paymentContentVisibility = if (paymentTransaction != null) View.VISIBLE else View.GONE
                paymentContentTextView.visibility = if (paymentTransaction != null) View.VISIBLE else View.GONE

                order.refundOrCancellationTransaction.also { refundOrCancellationTransaction ->
                    val refundAndCancelVisibility = if (refundOrCancellationTransaction == null && paymentTransaction != null)
                        View.VISIBLE else View.GONE
                    refundButton.visibility = refundAndCancelVisibility
                    cancelButton.visibility = refundAndCancelVisibility
                    refundOrCancellationContentTextView.text = refundOrCancellationTransaction
                        ?.getCompactRepresentation()
                    refundOrCancellationContentTextView.visibility = if (refundOrCancellationTransaction != null) View.VISIBLE else View.GONE
                }
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun onBackPressed() {
        finish()
    }
}
